#version 150

#ifdef VERTEX_SHADER

in vec3 in_position;
in mat4 in_instance_mat4;
in vec4 in_color;
in float in_depthFailScale;

out vec4 vs_color;
out float vs_depthFailScale;

uniform mat4 viewProjection;

void main() {
    vec4 model_position = vec4(in_position, 1.0f);
    gl_Position = viewProjection * (in_instance_mat4 * model_position);
    vs_color = in_color;
    vs_depthFailScale = in_depthFailScale;
}

#endif

#ifdef FRAGMENT_SHADER

in vec4 vs_color;
in float vs_depthFailScale;

out vec4 fragColor;

uniform sampler2D in_depth;

layout(std140) uniform ScreenBlock {
    float width;
    float height;
};

void main() {
    vec2 texcoord = gl_FragCoord.xy * vec2(1.0 / width, 1.0 / height);
    float depth = texture(in_depth, texcoord).r;

    // @TODO: this should be using linear depth so the constant offset makes sense!
    const float depthOffset = 0.0;

    fragColor = vs_color;
    fragColor.a *= 1.0 - (1.0 - vs_depthFailScale) * step(depth + depthOffset, gl_FragCoord.z);
}

#endif

