
#version 410

#ifdef VERTEX_SHADER

in vec3 in_position;

in int in_codepoint;
in vec3 in_glyphPosition;
in vec2 in_glyphDimensions;
in vec4 in_color;

out vec2 texcoord;
out vec4 color;

uniform sampler1D texcoordsMap;
uniform mat4 viewProjection;

uniform mat4 model;


void main() {
    
    vec3 modelPosition = in_glyphPosition;
    modelPosition.x += in_position.x * in_glyphDimensions.x;
    modelPosition.y += in_position.z * in_glyphDimensions.y;
    
    vec4 texcoords = texelFetch(texcoordsMap, in_codepoint, 0);
    vec2 texcoord0 = texcoords.xy;
    vec2 texcoord1 = texcoords.zw;

    gl_Position = viewProjection * (model * vec4(modelPosition, 1.0));
    texcoord = mix(texcoord0, texcoord1, in_position.xz);
    color = in_color;
}

#endif

#ifdef FRAGMENT_SHADER

in vec2 texcoord;
in vec4 color;

out vec4 fragColor;

uniform sampler2D atlas;

float median(float r, float g, float b) {
    return max(min(r, g), min(max(r, g), b));
}

void main() {
    vec4 s = texture(atlas, texcoord);

    float sigDist = median(s.r, s.g, s.b) - 0.3;
    float opacity = clamp(sigDist/fwidth(sigDist) + 0.5, 0.0, 1.0);

    fragColor = vec4(color.rgb, opacity);
}

#endif

