#version 150

#include "shaders/lib/camera.glsl"

#ifdef VERTEX_SHADER

in vec3 in_position;
in vec2 in_texcoord;

out vec3 vs_viewRay;
out vec2 vs_texcoord;

void main() {
    gl_Position = vec4(in_position.xz * 2 - 1, 0.0, 1.0);
    vs_texcoord = in_texcoord;

    vec3 viewPosition = (invProjection * vec4(gl_Position.xyz, 1.0)).xyz;
    vs_viewRay = vec3(viewPosition.xy / viewPosition.z, 1.0);
}

#endif

#ifdef FRAGMENT_SHADER

in vec3 vs_viewRay;
in vec2 vs_texcoord;

out vec4 fragColor;

uniform sampler2D in_diffuse;
uniform sampler2D in_depth;
uniform sampler2D in_normal;

uniform vec3 lightDirection;
uniform vec3 lightColor;

void main() {
    const float epsilon = 1e-6;
    float nonlinearDepth = texture(in_depth, vs_texcoord).x * 2 - 1;
    float linearDepth = -2.0 * nearClip / min(nonlinearDepth - 1.0, 1.0f - epsilon);

    vec3 viewPosition = vs_viewRay * linearDepth;
    vec3 viewNormal = texture(in_normal, vs_texcoord).xyz;

    vec3 diffuseTerm = texture(in_diffuse, vs_texcoord).rgb;
    float L = max(dot(-lightDirection, viewNormal), 0);

    fragColor.rgb = diffuseTerm * lightColor * L;
    fragColor.a = 1.0;
}

#endif

